@@ -1,3 +1,4 @@
+# encoding: utf-8
 """
 USAGE:
 
@@ -6,15 +7,21 @@ USAGE:
 
 ACTIONS:
  authorize      authorize the command-line tool to interact with Twitter
- follow         add the specified user to your follow list
+ follow         follow a user
  friends        get latest tweets from your friends (default action)
  help           print this help text that you are currently reading
- leave          remove the specified user from your following list
+ leave          stop following a user
+ list           get list of a user's lists; give a list name to get
+                    tweets from that list
+ mylist         get list of your lists; give a list name to get tweets
+                    from that list
  public         get latest public tweets
- replies        get latest replies
+ pyprompt       start a Python prompt for interacting with the twitter
+                    object directly
+ replies        get latest replies to you
  search         search twitter (Beware: octothorpe, escape it)
  set            set your twitter status
- shell          login the twitter shell
+ shell          login to the twitter shell
 
 
 OPTIONS:
@@ -28,8 +35,8 @@ OPTIONS:
  -l --length <count>        specify number of status updates shown
                             (default: 20, max: 200)
  -t --timestamp             show time before status lines
- -d --datestamp             shoe date before status lines
-    --no-ssl                use HTTP instead of more secure HTTPS
+ -d --datestamp             show date before status lines
+    --no-ssl                use less-secure HTTP instead of HTTPS
     --oauth <filename>      filename to read/store oauth credentials to
 
 FORMATS for the --format option
@@ -54,6 +61,8 @@ prompt: <twitter_shell_prompt e.g. '[cyan]twitter[R]> '>
  home directory.
 """
 
+from __future__ import print_function
+
 CONSUMER_KEY='uS6hO2sV6tDKIOeVjhnFnQ'
 CONSUMER_SECRET='MEYTOS97VvlHX7K1rwHPEqVpTSqZ71HtvoK4sVuYk'
 
@@ -63,15 +72,22 @@ from getopt import gnu_getopt as getopt, GetoptError
 from getpass import getpass
 import re
 import os.path
-from ConfigParser import SafeConfigParser
+try:
+    from ConfigParser import SafeConfigParser
+except ImportError:
+    from configparser import ConfigParser as SafeConfigParser
 import datetime
-from urllib import quote
+try:
+    from urllib.parse import quote
+except ImportError:
+    from urllib2 import quote
 import webbrowser
 
-from api import Twitter, TwitterError
-from oauth import OAuth, write_token_file, read_token_file
-from oauth_dance import oauth_dance
-import ansi
+from .api import Twitter, TwitterError
+from .oauth import OAuth, write_token_file, read_token_file
+from .oauth_dance import oauth_dance
+from . import ansi
+from .util import smrt_input, printNicely
 
 OPTIONS = {
     'action': 'friends',
@@ -141,7 +157,7 @@ def get_time_string(status, options, format="%a %b %d %H:%M:%S +0000 %Y"):
 
 class StatusFormatter(object):
     def __call__(self, status, options):
-        return (u"%s%s %s" %(
+        return ("%s%s %s" %(
             get_time_string(status, options),
             status['user']['screen_name'], status['text']))
 
@@ -151,14 +167,14 @@ class AnsiStatusFormatter(object):
 
     def __call__(self, status, options):
         colour = self._colourMap.colourFor(status['user']['screen_name'])
-        return (u"%s%s%s%s %s" %(
+        return ("%s%s%s%s %s" %(
             get_time_string(status, options),
             ansi.cmdColour(colour), status['user']['screen_name'],
             ansi.cmdReset(), status['text']))
 
 class VerboseStatusFormatter(object):
     def __call__(self, status, options):
-        return (u"-- %s (%s) on %s\n%s\n" %(
+        return ("-- %s (%s) on %s\n%s\n" %(
             status['user']['screen_name'],
             status['user']['location'],
             status['created_at'],
@@ -168,19 +184,44 @@ class URLStatusFormatter(object):
     urlmatch = re.compile(r'https?://\S+')
     def __call__(self, status, options):
         urls = self.urlmatch.findall(status['text'])
-        return u'\n'.join(urls) if urls else ""
+        return '\n'.join(urls) if urls else ""
+
+
+class ListsFormatter(object):
+    def __call__(self, list):
+        if list['description']:
+            list_str = "%-30s (%s)" % (list['name'], list['description'])
+        else:
+            list_str = "%-30s" % (list['name'])
+        return "%s\n" % list_str
+
+class ListsVerboseFormatter(object):
+    def __call__(self, list):
+        list_str = "%-30s\n description: %s\n members: %s\n mode:%s\n" % (list['name'], list['description'], list['member_count'], list['mode'])
+        return list_str
+
+class AnsiListsFormatter(object):
+    def __init__(self):
+        self._colourMap = ansi.ColourMap()
+
+    def __call__(self, list):
+        colour = self._colourMap.colourFor(list['name'])
+        return ("%s%-15s%s %s" %(
+            ansi.cmdColour(colour), list['name'],
+            ansi.cmdReset(), list['description']))
+
 
 class AdminFormatter(object):
     def __call__(self, action, user):
-        user_str = u"%s (%s)" %(user['screen_name'], user['name'])
+        user_str = "%s (%s)" %(user['screen_name'], user['name'])
         if action == "follow":
-            return u"You are now following %s.\n" %(user_str)
+            return "You are now following %s.\n" %(user_str)
         else:
-            return u"You are no longer following %s.\n" %(user_str)
+            return "You are no longer following %s.\n" %(user_str)
 
 class VerboseAdminFormatter(object):
     def __call__(self, action, user):
-        return(u"-- %s: %s (%s): %s" % (
+        return("-- %s: %s (%s): %s" % (
             "Following" if action == "follow" else "Leaving",
             user['screen_name'],
             user['name'],
@@ -188,7 +229,7 @@ class VerboseAdminFormatter(object):
 
 class SearchFormatter(object):
     def __call__(self, result, options):
-        return(u"%s%s %s" %(
+        return("%s%s %s" %(
             get_time_string(result, options, "%a, %d %b %Y %H:%M:%S +0000"),
             result['from_user'], result['text']))
 
@@ -199,7 +240,7 @@ class URLSearchFormatter(object):
     urlmatch = re.compile(r'https?://\S+')
     def __call__(self, result, options):
         urls = self.urlmatch.findall(result['text'])
-        return u'\n'.join(urls) if urls else ""
+        return '\n'.join(urls) if urls else ""
 
 class AnsiSearchFormatter(object):
     def __init__(self):
@@ -207,7 +248,7 @@ class AnsiSearchFormatter(object):
 
     def __call__(self, result, options):
         colour = self._colourMap.colourFor(result['from_user'])
-        return (u"%s%s%s%s %s" %(
+        return ("%s%s%s%s %s" %(
             get_time_string(result, options, "%a, %d %b %Y %H:%M:%S +0000"),
             ansi.cmdColour(colour), result['from_user'],
             ansi.cmdReset(), result['text']))
@@ -248,6 +289,14 @@ search_formatters = {
 }
 formatters['search'] = search_formatters
 
+lists_formatters = {
+    'default': ListsFormatter,
+    'verbose': ListsVerboseFormatter,
+    'urls': None,
+    'ansi': AnsiListsFormatter
+}
+formatters['lists'] = lists_formatters
+
 def get_formatter(action_type, options):
     formatters_dict = formatters.get(action_type)
     if (not formatters_dict):
@@ -274,13 +323,13 @@ class Action(object):
 
         prompt = 'You really want to %s %s? ' %(subject, sample)
         try:
-            answer = raw_input(prompt).lower()
+            answer = input(prompt).lower()
             if careful:
                 return answer in ('yes', 'y')
             else:
                 return answer not in ('no', 'n')
         except EOFError:
-            print >>sys.stderr # Put Newline since Enter was never pressed
+            print(file=sys.stderr) # Put Newline since Enter was never pressed
             # TODO:
                 #   Figure out why on OS X the raw_input keeps raising
                 #   EOFError and is never able to reset and get more input
@@ -301,7 +350,7 @@ class Action(object):
             else:
                 doAction()
         except KeyboardInterrupt:
-            print >>sys.stderr, '\n[Keyboard Interrupt]'
+            print('\n[Keyboard Interrupt]', file=sys.stderr)
             pass
 
 class NoSuchActionError(Exception):
@@ -311,12 +360,6 @@ class NoSuchAction(Action):
     def __call__(self, twitter, options):
         raise NoSuchActionError("No such action: %s" %(options['action']))
 
-def printNicely(string):
-    if sys.stdout.encoding:
-        print string.encode(sys.stdout.encoding, 'replace')
-    else:
-        print string.encode('utf-8')
-
 class StatusAction(Action):
     def __call__(self, twitter, options):
         statuses = self.getStatuses(twitter, options)
@@ -335,7 +378,7 @@ class SearchAction(Action):
         # We need to bypass the TwitterCall parameter encoding, so we
         # don't encode the plus sign, so we have to encode it ourselves
         query_string = "+".join(
-            [quote(term.decode(get_term_encoding()))
+            [quote(term)
              for term in options['extra_args']])
 
         results = twitter.search(q=query_string)['results']
@@ -352,17 +395,44 @@ class AdminAction(Action):
         af = get_formatter('admin', options)
         try:
             user = self.getUser(twitter, options['extra_args'][0])
-        except TwitterError, e:
-            print "There was a problem following or leaving the specified user."
-            print "You may be trying to follow a user you are already following;"
-            print "Leaving a user you are not currently following;"
-            print "Or the user may not exist."
-            print "Sorry."
-            print
-            print e
+        except TwitterError as e:
+            print("There was a problem following or leaving the specified user.")
+            print("You may be trying to follow a user you are already following;")
+            print("Leaving a user you are not currently following;")
+            print("Or the user may not exist.")
+            print("Sorry.")
+            print()
+            print(e)
         else:
             printNicely(af(options['action'], user))
 
+class ListsAction(StatusAction):
+    def getStatuses(self, twitter, options):
+        if not options['extra_args']:
+            raise TwitterError("Please provide a user to query for lists")
+
+        screen_name = options['extra_args'][0]
+
+        if not options['extra_args'][1:]:
+            lists = twitter.user.lists(user=screen_name)['lists']
+            if not lists:
+                printNicely("This user has no lists.")
+            for list in lists:
+                lf = get_formatter('lists', options)
+                printNicely(lf(list))
+            return []
+        else:
+            return reversed(twitter.user.lists.list.statuses(
+                    user=screen_name, list=options['extra_args'][1]))
+
+
+class MyListsAction(ListsAction):
+    def getStatuses(self, twitter, options):
+        screen_name = twitter.account.verify_credentials()['screen_name']
+        options['extra_args'].insert(0, screen_name)
+        return ListsAction.getStatuses(self, twitter, options)
+
+
 class FriendsAction(StatusAction):
     def getStatuses(self, twitter, options):
         return reversed(twitter.statuses.friends_timeline(count=options["length"]))
@@ -385,11 +455,10 @@ class LeaveAction(AdminAction):
 
 class SetStatusAction(Action):
     def __call__(self, twitter, options):
-        statusTxt = (" ".join(options['extra_args']).decode(get_term_encoding())
+        statusTxt = (" ".join(options['extra_args'])
                      if options['extra_args']
-                     else unicode(raw_input("message: ")))
-        status = (statusTxt.encode('utf8', 'replace'))
-        twitter.statuses.update(status=status)
+                     else str(input("message: ")))
+        twitter.statuses.update(status=statusTxt)
 
 class TwitterShell(Action):
 
@@ -408,39 +477,47 @@ class TwitterShell(Action):
         while True:
             options['action'] = ""
             try:
-                args = raw_input(prompt).split()
+                args = input(prompt).split()
                 parse_args(args, options)
                 if not options['action']:
                     continue
                 elif options['action'] == 'exit':
                     raise SystemExit(0)
                 elif options['action'] == 'shell':
-                    print >>sys.stderr, 'Sorry Xzibit does not work here!'
+                    print('Sorry Xzibit does not work here!', file=sys.stderr)
                     continue
                 elif options['action'] == 'help':
-                    print >>sys.stderr, '''\ntwitter> `action`\n
+                    print('''\ntwitter> `action`\n
                           The Shell Accepts all the command line actions along with:
 
                           exit    Leave the twitter shell (^D may also be used)
 
-                          Full CMD Line help is appended below for your convinience.'''
+                          Full CMD Line help is appended below for your convinience.''', file=sys.stderr)
                 Action()(twitter, options)
                 options['action'] = ''
-            except NoSuchActionError, e:
-                print >>sys.stderr, e
+            except NoSuchActionError as e:
+                print(e, file=sys.stderr)
             except KeyboardInterrupt:
-                print >>sys.stderr, '\n[Keyboard Interrupt]'
+                print('\n[Keyboard Interrupt]', file=sys.stderr)
             except EOFError:
-                print >>sys.stderr
+                print(file=sys.stderr)
                 leaving = self.ask(subject='Leave')
                 if not leaving:
-                    print >>sys.stderr, 'Excellent!'
+                    print('Excellent!', file=sys.stderr)
                 else:
                     raise SystemExit(0)
 
+class PythonPromptAction(Action):
+    def __call__(self, twitter, options):
+        try:
+            while True:
+                smrt_input(globals(), locals())
+        except EOFError:
+            pass
+
 class HelpAction(Action):
     def __call__(self, twitter, options):
-        print __doc__
+        print(__doc__)
 
 class DoNothingAction(Action):
     def __call__(self, twitter, options):
@@ -450,9 +527,12 @@ actions = {
     'authorize' : DoNothingAction,
     'follow'    : FollowAction,
     'friends'   : FriendsAction,
+    'list'      : ListsAction,
+    'mylist'    : MyListsAction,
     'help'      : HelpAction,
     'leave'     : LeaveAction,
     'public'    : PublicAction,
+    'pyprompt'  : PythonPromptAction,
     'replies'   : RepliesAction,
     'search'    : SearchAction,
     'set'       : SetStatusAction,
@@ -473,9 +553,9 @@ def main(args=sys.argv[1:]):
     arg_options = {}
     try:
         parse_args(args, arg_options)
-    except GetoptError, e:
-        print >> sys.stderr, "I can't do that, %s." %(e)
-        print >> sys.stderr
+    except GetoptError as e:
+        print("I can't do that, %s." %(e), file=sys.stderr)
+        print(file=sys.stderr)
         raise SystemExit(1)
 
     config_path = os.path.expanduser(
@@ -487,13 +567,13 @@ def main(args=sys.argv[1:]):
     # arguments.
     options = dict(OPTIONS)
     for d in config_options, arg_options:
-        for k,v in d.items():
+        for k,v in list(d.items()):
             if v: options[k] = v
 
     if options['refresh'] and options['action'] not in (
         'friends', 'public', 'replies'):
-        print >> sys.stderr, "You can only refresh the friends, public, or replies actions."
-        print >> sys.stderr, "Use 'twitter -h' for help."
+        print("You can only refresh the friends, public, or replies actions.", file=sys.stderr)
+        print("Use 'twitter -h' for help.", file=sys.stderr)
         return 1
 
     oauth_filename = os.path.expanduser(options['oauth_filename'])
@@ -515,10 +595,10 @@ def main(args=sys.argv[1:]):
 
     try:
         Action()(twitter, options)
-    except NoSuchActionError, e:
-        print >>sys.stderr, e
+    except NoSuchActionError as e:
+        print(e, file=sys.stderr)
         raise SystemExit(1)
-    except TwitterError, e:
-        print >> sys.stderr, str(e)
-        print >> sys.stderr, "Use 'twitter -h' for help."
+    except TwitterError as e:
+        print(str(e), file=sys.stderr)
+        print("Use 'twitter -h' for help.", file=sys.stderr)
         raise SystemExit(1)
